@@ -1,22 +1,29 @@
-/*
- Copyright (C) 2015 Bert Frees
+/* liblouis Braille Translation and Back-Translation Library
+
+Copyright (C) 2015 Bert Frees
  
- This file is free software; you can redistribute it and/or modify it under
- the terms of the Lesser or Library GNU General Public License as published by
- the Free Software Foundation; either version 3, or (at your option) any later
- version.
- 
- This file is distributed in the hope that it will be useful, but WITHOUT ANY
- WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
- A PARTICULAR PURPOSE.  See the Library GNU General Public License for more
- details.
- 
- You should have received a copy of the Library GNU General Public License
- along with this program; see the file COPYING.  If not, write to the Free
- Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- USA.
+This file is part of liblouis.
+
+liblouis is free software: you can redistribute it and/or modify it
+under the terms of the GNU Lesser General Public License as published
+by the Free Software Foundation, either version 2.1 of the License, or
+(at your option) any later version.
+
+liblouis is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with liblouis. If not, see <http://www.gnu.org/licenses/>.
+
 */
 
+/**
+ * @file
+ * @brief Find translation tables
+ */
+
 #include <config.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,7 +42,7 @@ typedef struct List
   struct List * tail;
 } List;
 
-/*
+/**
  * Returns a list with the element `x' added to `list'. Returns a sorted list
  * if `cmp' is not NULL and if `list' is also sorted. New elements replace
  * existing ones if they are equal according to `cmp'. If `cmp' is NULL,
@@ -100,7 +107,7 @@ list_conj(List * list,
     }
 }
 
-/*
+/**
  * Free an instance of type List.
  */
 static void
@@ -115,7 +122,7 @@ list_free(List * list)
     }
 }
 
-/*
+/**
  * Sort a list based on a comparison function.
  */
 static List *
@@ -132,7 +139,7 @@ list_sort(List * list, int (* cmp)(void *, void *))
   return newList;
 }
 
-/*
+/**
  * Get the size of a list.
  */
 static int
@@ -145,7 +152,7 @@ list_size(List * list)
   return len;
 }
 
-/*
+/**
  * Convert a list into a NULL terminated array.
  */
 static void **
@@ -182,9 +189,10 @@ typedef struct
   List * features;
 } TableMeta;
 
-/*
- * Create an instance of type Feature. The `key' and `val' strings are
- * duplicated. Leaving out the `val' argument results in a value of "yes".
+/**
+ * Create an instance of type Feature.
+ * The `key' and `val' strings are duplicated. Leaving out the `val'
+ * argument results in a value of "yes".
  */
 static Feature
 feature_new(const char * key, const char * val)
@@ -196,7 +204,7 @@ feature_new(const char * key, const char * val)
   return f;
 }
 
-/*
+/**
  * Free an instance of type Feature
  */
 static void
@@ -212,7 +220,7 @@ feature_free(Feature * f)
 
 /* ======================================================================== */
 
-/*
+/**
  * Sort features based on their keys.
  */
 static int
@@ -221,16 +229,17 @@ cmpKeys(Feature * f1, Feature * f2)
   return strcmp(f1->key, f2->key);
 }
 
-/*
- * Compute the match quotient of the features in a query against the features
- * in a table's metadata. The features are assumed to be sorted and to have no
- * duplicate keys. The query's features must be of type
- * FeatureWithImportance. How a feature contributes to the match quotient
- * depends on its importance, on whether the feature is undefined, defined
- * with the same value (positive match), or defined with a different value
- * (negative match), and on the `fuzzy' argument. If the `fuzzy' argument
- * evaluates to true, negative matches and undefined features get a lower
- * penalty.
+/**
+ * Compute the match quotient of the features in a query against the features in a table's metadata.
+ *
+ * The features are assumed to be sorted and to have no duplicate
+ * keys. The query's features must be of type FeatureWithImportance.
+ * How a feature contributes to the match quotient depends on its
+ * importance, on whether the feature is undefined, defined with the
+ * same value (positive match), or defined with a different value
+ * (negative match), and on the `fuzzy' argument. If the `fuzzy'
+ * argument evaluates to true, negative matches and undefined features
+ * get a lower penalty.
  */
 static int
 matchFeatureLists(const List * query, const List * tableFeatures, int fuzzy)
@@ -302,7 +311,7 @@ matchFeatureLists(const List * query, const List * tableFeatures, int fuzzy)
   return quotient;
 }
 
-/*
+/**
  * Return true if a character matches [0-9A-Za-z_-]
  */
 static int
@@ -315,7 +324,7 @@ isIdentChar(char c)
       || c == '_';
 }
 
-/*
+/**
  * Parse a table query into a list of features. Features defined first get a
  * higher importance.
  */
@@ -395,7 +404,7 @@ parseQuery(const char * query)
   return NULL;
 }
 
-/*
+/**
  * Convert a widechar string to a normal string.
  */
 static char *
@@ -409,7 +418,7 @@ widestrToStr(const widechar * str, size_t n)
   return result;
 }
 
-/*
+/**
  * Extract a list of features from a table.
  */
 static List *
@@ -550,7 +559,7 @@ lou_indexTables(const char ** tables)
 #define DIR_SEP '/'
 #endif
 
-/*
+/**
  * Returns the list of files found on searchPath, where searchPath is a
  * comma-separated list of directories.
  */
@@ -606,7 +615,7 @@ lou_findTable(const char * query)
       logMessage(LOG_WARN, "Tables have not been indexed yet. Indexing LOUIS_TABLEPATH.");
       searchPath = getTablePath();
       tables = listFiles(searchPath);
-      tablesArray = list_toArray(tables, NULL);
+      tablesArray = (const char **)list_toArray(tables, NULL);
       lou_indexTables(tablesArray);
       free(searchPath);
       list_free(tables);
