@@ -22,7 +22,7 @@ main(int argc, char **argv)
     char* table = "pass0_typebuf.ctb";
     char* text = "foo baz";
     char* expected = "foobar .baz";
-    char* typeform = malloc(20 * sizeof(char));
+    formtype* typeform = malloc(20 * sizeof(formtype));
     for (i = 0; i < 7; i++)
       typeform[i] = 0;
     for (i = 4; i < 7; i++)
