@@ -22,7 +22,7 @@
 
    */
 
-# include <config.h>
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -30,6 +30,7 @@
 #include "liblouis.h"
 #include "louis.h"
 #include "progname.h"
+#include "unistr.h"
 #include "version-etc.h"
 
 #define BUFSIZE MAXSTRING - 4
@@ -37,6 +38,8 @@
 static int forward_flag = 0;
 static int backward_flag = 0;
 
+static FILE *input;
+
 static const struct option longopts[] =
 {
   { "help", no_argument, NULL, 'h' },
@@ -56,6 +59,7 @@ translate_input (int forward_translation, char *table_name)
 {
   char charbuf[BUFSIZE];
   char *outputbuf;
+  size_t outlen;
   widechar inbuf[BUFSIZE];
   widechar transbuf[BUFSIZE];
   int inlen;
@@ -67,7 +71,7 @@ translate_input (int forward_translation, char *table_name)
     {
       translen = BUFSIZE;
       k = 0;
-      while ((ch = getchar ()) != '\n' && ch != EOF && k < BUFSIZE)
+      while ((ch = fgetc(input)) != '\n' && ch != EOF && k < BUFSIZE)
 	charbuf[k++] = ch;
       if (ch == EOF && k == 0)
 	break;
@@ -81,10 +85,13 @@ translate_input (int forward_translation, char *table_name)
 					  transbuf, &translen, NULL, NULL, 0);
       if (!result)
 	break;
-      outputbuf = showString (transbuf, translen);
-      k = strlen (outputbuf) - 1;
-      outputbuf[k] = 0;
-      printf ("%s\n", &outputbuf[1]);
+#ifdef WIDECHARS_ARE_UCS4
+      outputbuf = u32_to_u8(transbuf, translen, NULL, &outlen);
+#else
+      outputbuf = u16_to_u8(transbuf, translen, NULL, &outlen);
+#endif
+      printf ("%.*s\n", (int)outlen, outputbuf);
+      free(outputbuf);
     }
   lou_free ();
 }
@@ -159,9 +166,9 @@ main (int argc, char **argv)
       exit (EXIT_FAILURE);
     }
 
-  if (optind != argc - 1)
+  /*if (optind != argc - 1)
     {
-      /* Print error message and exit.  */
+      /* Print error message and exit.  *
       if (optind < argc - 1)
 	fprintf (stderr, "%s: extra operand: %s\n",
 		 program_name, argv[optind + 1]);
@@ -171,7 +178,21 @@ main (int argc, char **argv)
       fprintf (stderr, "Try `%s --help' for more information.\n",
                program_name);
       exit (EXIT_FAILURE);
-    }
+    }*/
+	if(!argv[optind + 0])
+	{
+	fprintf (stderr, "%s: no table specified\n", 
+		 program_name);
+      fprintf (stderr, "Try `%s --help' for more information.\n",
+               program_name);
+      exit (EXIT_FAILURE);
+      }
+	if(argv[optind + 1])
+	{
+		input = fopen(argv[optind + 1], "r");
+	}
+	else
+		input = stdin;
 
   /* assume forward translation by default */
   translate_input (!backward_flag, argv[optind]);
